@@ -13,22 +13,19 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Generation of file-ids."""
 
-from bzrlib.errors import RevisionNotPresent, NotBranchError
-from bzrlib.inventory import ROOT_ID
+from bzrlib import osutils, ui
+from bzrlib.errors import NotBranchError, RevisionNotPresent
 from bzrlib.knit import KnitVersionedFile
+from bzrlib.revision import NULL_REVISION
 from bzrlib.trace import mutter
-from bzrlib.transport import get_transport
-from bzrlib.ui import ui_factory
-import bzrlib.urlutils as urlutils
-from warnings import warn
 
-import os
+import urllib
+
 import sha
 
-import logwalker
-from repository import (escape_svn_path, generate_svn_revision_id, 
-                        parse_svn_revision_id, MAPPING_VERSION)
+from revids import escape_svn_path
 
 def generate_svn_file_id(uuid, revnum, branch, path):
     """Create a file id identifying a Subversion file.
@@ -38,29 +35,23 @@ def generate_svn_file_id(uuid, revnum, branch, path):
     :param branch: Branch path of the branch in which the file was introduced.
     :param path: Original path of the file within the branch
     """
-    if path == "":
-        return ROOT_ID
-    introduced_revision_id = generate_svn_revision_id(uuid, revnum, branch)
-    ret = "%s-%s" % (introduced_revision_id, escape_svn_path(path))
+    ret = "%d@%s:%s:%s" % (revnum, uuid, escape_svn_path(branch), escape_svn_path(path))
     if len(ret) > 150:
-        basename = os.path.basename(path)
-        parent = path[:-len(basename)]
-        ret = "%s-%s-%s" % (introduced_revision_id, 
-                            sha.new(parent).hexdigest(),
-                            escape_svn_path(basename))
-    return ret
+        ret = "%d@%s:%s;%s" % (revnum, uuid, 
+                            escape_svn_path(branch),
+                            sha.new(path).hexdigest())
+    assert isinstance(ret, str)
+    return osutils.safe_file_id(ret)
 
 
-def generate_file_id(revid, path):
-    (uuid, branch, revnum) = parse_svn_revision_id(revid)
-    return generate_svn_file_id(uuid, revnum, branch, path)
+def generate_file_id(repos, revid, path):
+    (branch, revnum, _) = repos.lookup_revision_id(revid)
+    return generate_svn_file_id(repos.uuid, revnum, branch, path)
 
 
-def get_local_changes(paths, scheme, uuid, get_children=None):
+def get_local_changes(paths, scheme, generate_revid, get_children=None):
     new_paths = {}
-    names = paths.keys()
-    names.sort()
-    for p in names:
+    for p in sorted(paths.keys()):
         data = paths[p]
         new_p = scheme.unprefix(p)[1]
         if data[1] is not None:
@@ -71,8 +62,8 @@ def get_local_changes(paths, scheme, uuid, get_children=None):
                 if (crp == "" and new_p == ""):
                     data = ('M', None, None)
                 else:
-                    data = (data[0], crp, generate_svn_revision_id(
-                        uuid, data[2], cbp))
+                    data = (data[0], crp, generate_revid(
+                                  data[2], cbp.encode("utf-8"), str(scheme)))
             except NotBranchError:
                 # Copied from outside of a known branch
                 # Make it look like the files were added in this revision
@@ -85,8 +76,8 @@ def get_local_changes(paths, scheme, uuid, get_children=None):
         new_paths[new_p] = data
     return new_paths
 
-dbs = {}
 
+FILEIDMAP_VERSION = 1
 
 class FileIdMap(object):
     """ File id store. 
@@ -95,92 +86,107 @@ class FileIdMap(object):
 
     revnum -> branch -> path -> fileid
     """
-    def __init__(self, repos, cache_db):
+    def __init__(self, repos, cache_transport):
         self.repos = repos
-        self.cachedb = cache_db
-        self.cachedb.executescript("""
-        create table if not exists filemap (filename text, id integer, create_revid text, revid text);
-        create index if not exists revid on filemap(revid);
-        """)
-        self.cachedb.commit()
+        self.idmap_knit = KnitVersionedFile("fileidmap-v%d" % FILEIDMAP_VERSION, cache_transport, create=True)
 
     def save(self, revid, parent_revids, _map):
         mutter('saving file id map for %r' % revid)
-        for filename in _map:
-            self.cachedb.execute("insert into filemap (filename, id, create_revid, revid) values(?,?,?,?)", (filename, _map[filename][0], _map[filename][1], revid))
-        self.cachedb.commit()
+                
+        self.idmap_knit.add_lines_with_ghosts(revid, parent_revids, 
+                ["%s\t%s\t%s\n" % (urllib.quote(filename), urllib.quote(_map[filename][0]), 
+                                        urllib.quote(_map[filename][1])) for filename in sorted(_map.keys())])
 
     def load(self, revid):
         map = {}
-        for filename,create_revid,id in self.cachedb.execute("select filename, create_revid, id from filemap where revid='%s'"%revid):
-            map[filename] = (id.encode("utf-8"),create_revid.encode("utf-8"))
+        for line in self.idmap_knit.get_lines(revid):
+            (filename, id, create_revid) = line.rstrip("\n").split("\t", 3)
+            map[urllib.unquote(filename)] = (urllib.unquote(id), urllib.unquote(create_revid))
+            assert isinstance(map[urllib.unquote(filename)][0], str)
 
         return map
 
     def apply_changes(self, uuid, revnum, branch, global_changes, 
-                      renames, find_children=None):
+                      renames, scheme, find_children=None):
         """Change file id map to incorporate specified changes.
 
         :param uuid: UUID of repository changes happen in
         :param revnum: Revno for revision in which changes happened
         :param branch: Branch path where changes happened
         :param global_changes: Dict with global changes that happened
+        :param renames: List of renames
+        :param scheme: Branching scheme
         """
-        changes = get_local_changes(global_changes, self.repos.scheme,
-                                        uuid, find_children)
+        changes = get_local_changes(global_changes, scheme,
+                    self.repos.generate_revision_id, find_children)
         if find_children is not None:
             def get_children(path, revid):
-                (bp, revnum) = self.repos.parse_revision_id(revid)
+                (bp, revnum, scheme) = self.repos.lookup_revision_id(revid)
                 for p in find_children(bp+"/"+path, revnum):
-                    yield self.repos.scheme.unprefix(p)[1]
+                    yield scheme.unprefix(p)[1]
         else:
             get_children = None
 
-        revid = generate_svn_revision_id(uuid, revnum, branch)
+        revid = self.repos.generate_revision_id(revnum, branch, str(scheme))
 
         def new_file_id(x):
             if renames.has_key(x):
                 return renames[x]
-            return generate_file_id(revid, x)
+            return generate_file_id(self.repos, revid, x)
          
         return self._apply_changes(new_file_id, changes, get_children)
 
-    def get_map(self, uuid, revnum, branch, renames_cb):
+    def get_map(self, uuid, revnum, branch, renames_cb, scheme):
         """Make sure the map is up to date until revnum."""
         # First, find the last cached map
         todo = []
         next_parent_revs = []
-        map = {"": (ROOT_ID, None)} # No history -> empty map
-        for (bp, paths, rev) in self.repos.follow_branch_history(branch, revnum):
-            revid = generate_svn_revision_id(uuid, rev, bp)
-            map = self.load(revid)
-            if map != {}:
+        if revnum == 0:
+            assert branch == ""
+            return {"": (generate_svn_file_id(uuid, revnum, branch, ""), 
+              self.repos.generate_revision_id(revnum, branch, str(scheme)))}
+
+        # No history -> empty map
+        for (bp, paths, rev) in self.repos.follow_branch_history(branch, 
+                                             revnum, scheme):
+            revid = self.repos.generate_revision_id(rev, bp.encode("utf-8"), 
+                                                    str(scheme))
+            try:
+                map = self.load(revid)
                 # found the nearest cached map
                 next_parent_revs = [revid]
                 break
-            todo.append((revid, paths))
-    
+            except RevisionNotPresent:
+                todo.append((revid, paths))
+   
         # target revision was present
         if len(todo) == 0:
             return map
-    
-        if len(map.keys()) == 0:
-            map = {"": (ROOT_ID, None)} # No history -> empty map
-        todo.reverse()
-        
-        pb = ui_factory.nested_progress_bar()
+
+        if len(next_parent_revs) == 0:
+            if scheme.is_branch(""):
+                map = {"": (generate_svn_file_id(uuid, 0, "", ""), NULL_REVISION)}
+            else:
+                map = {}
+
+        pb = ui.ui_factory.nested_progress_bar()
 
         try:
             i = 1
-            for (revid, global_changes) in todo:
-                changes = get_local_changes(global_changes, self.repos.scheme,
-                                            uuid, self.repos._log.find_children)
+            for (revid, global_changes) in reversed(todo):
+                expensive = False
+                def log_find_children(path, revnum):
+                    expensive = True
+                    return self.repos._log.find_children(path, revnum)
+                changes = get_local_changes(global_changes, scheme,
+                                            self.repos.generate_revision_id, 
+                                            log_find_children)
                 pb.update('generating file id map', i, len(todo))
 
                 def find_children(path, revid):
-                    (bp, revnum) = self.repos.parse_revision_id(revid)
-                    for p in self.repos._log.find_children(bp+"/"+path, revnum):
-                        yield self.repos.scheme.unprefix(p)[1]
+                    (bp, revnum, scheme) = self.repos.lookup_revision_id(revid)
+                    for p in log_find_children(bp+"/"+path, revnum):
+                        yield scheme.unprefix(p)[1]
 
                 parent_revs = next_parent_revs
 
@@ -189,14 +195,14 @@ class FileIdMap(object):
                 def new_file_id(x):
                     if renames.has_key(x):
                         return renames[x]
-                    return generate_file_id(revid, x)
+                    return generate_file_id(self.repos, revid, x)
                 
                 revmap = self._apply_changes(new_file_id, changes, find_children)
                 for p in changes:
                     if changes[p][0] == 'M' and not revmap.has_key(p):
                         revmap[p] = map[p][0]
 
-                map.update(dict([(x, (revmap[x], revid)) for x in revmap]))
+                map.update(dict([(x, (str(revmap[x]), revid)) for x in revmap]))
 
                 # Mark all parent paths as changed
                 for p in revmap:
@@ -208,11 +214,16 @@ class FileIdMap(object):
                             break
                         map[parent] = map[parent][0], revid
                         
+                saved = False
+                if i % 500 == 0 or expensive:
+                    self.save(revid, parent_revs, map)
+                    saved = True
                 next_parent_revs = [revid]
                 i += 1
         finally:
             pb.finished()
-        self.save(revid, parent_revs, map)
+        if not saved:
+            self.save(revid, parent_revs, map)
         return map
 
 
@@ -220,9 +231,7 @@ class SimpleFileIdMap(FileIdMap):
     @staticmethod
     def _apply_changes(new_file_id, changes, find_children=None):
         map = {}
-        sorted_paths = changes.keys()
-        sorted_paths.sort()
-        for p in sorted_paths:
+        for p in sorted(changes.keys()):
             data = changes[p]
 
             if data[0] in ('A', 'R'):
