@@ -13,22 +13,32 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Subversion BzrDir formats."""
 
-from bzrlib.bzrdir import BzrDirFormat, BzrDir
-from bzrlib.errors import NotBranchError, NotLocalUrl, NoRepositoryPresent
+from bzrlib import urlutils
+from bzrlib.bzrdir import BzrDirFormat, BzrDir, format_registry
+from bzrlib.errors import (NotBranchError, NotLocalUrl, NoRepositoryPresent,
+                           NoWorkingTree, AlreadyBranchError)
 from bzrlib.lockable_files import TransportLock
-from bzrlib.ui import ui_factory
 from bzrlib.transport.local import LocalTransport
-import bzrlib.urlutils as urlutils
 
 from svn.core import SubversionException
 import svn.core, svn.repos
 
-from branch import SvnBranch
+from errors import NoSvnRepositoryPresent
 from repository import SvnRepository
-from scheme import BranchingScheme
 from transport import SvnRaTransport, bzr_to_svn_url, get_svn_ra_transport
 
+def get_rich_root_format():
+    format = BzrDirFormat.get_default_format()
+    if format.repository_format.rich_root_data:
+        return format
+    # Default format does not support rich root data, 
+    # fall back to dirstate-with-subtree
+    format = format_registry.make_bzrdir('dirstate-with-subtree')
+    assert format.repository_format.rich_root_data
+    return format
+
 
 class SvnRemoteAccess(BzrDir):
     """BzrDir implementation for Subversion connections.
@@ -36,10 +46,12 @@ class SvnRemoteAccess(BzrDir):
     This is used for all non-checkout connections 
     to Subversion repositories.
     """
-    def __init__(self, _transport, _format, scheme=None):
+    def __init__(self, _transport, _format):
         """See BzrDir.__init__()."""
         _transport = get_svn_ra_transport(_transport)
-        super(SvnRemoteAccess, self).__init__(_transport, _format)
+        self._format = _format
+        self.transport = None
+        self.root_transport = _transport
 
         svn_url = bzr_to_svn_url(self.root_transport.base)
         self.svn_root_url = _transport.get_repos_root()
@@ -47,49 +59,43 @@ class SvnRemoteAccess(BzrDir):
         assert svn_url.startswith(self.svn_root_url)
         self.branch_path = svn_url[len(self.svn_root_url):]
 
-        if scheme is None:
-            self.scheme = BranchingScheme.guess_scheme(self.branch_path)
-        else:
-            self.scheme = scheme
-
-        if (not self.scheme.is_branch(self.branch_path) and 
-                self.branch_path != ""):
-            raise NotBranchError(path=self.root_transport.base)
-
-    def clone(self, url, revision_id=None, basis=None, force_new_repo=False):
+    def clone(self, url, revision_id=None, force_new_repo=False):
         """See BzrDir.clone().
 
         Not supported on Subversion connections.
         """
         raise NotImplementedError(SvnRemoteAccess.clone)
 
-    def sprout(self, url, revision_id=None, basis=None, force_new_repo=False):
+    def sprout(self, url, revision_id=None, force_new_repo=False,
+            recurse='down', possible_transports=None):
         """See BzrDir.sprout()."""
-        result = BzrDirFormat.get_default_format().initialize(url)
+        # FIXME: Use possible_transports
+        # FIXME: Use recurse
+        format = get_rich_root_format()
+        result = format.initialize(url)
         repo = self.find_repository()
         if force_new_repo:
-            result_repo = repo.clone(result, revision_id, basis)
+            result_repo = repo.clone(result, revision_id)
         else:
             try:
                 result_repo = result.find_repository()
                 result_repo.fetch(repo, revision_id=revision_id)
             except NoRepositoryPresent:
-                result_repo = repo.clone(result, revision_id, basis)
-
+                result_repo = repo.clone(result, revision_id)
         branch = self.open_branch()
         result_branch = branch.sprout(result, revision_id)
         if result_branch.repository.make_working_trees():
             result.create_workingtree()
         return result
 
-    def open_repository(self):
+    def open_repository(self, _unsupported=False):
         """Open the repository associated with this BzrDir.
         
         :return: instance of SvnRepository.
         """
         if self.branch_path == "":
             return SvnRepository(self, self.root_transport)
-        raise NoRepositoryPresent(self)
+        raise NoSvnRepositoryPresent(self.root_transport.base)
 
     def find_repository(self):
         """Open the repository associated with this BzrDir.
@@ -98,17 +104,18 @@ class SvnRemoteAccess(BzrDir):
         """
         transport = self.root_transport
         if self.svn_root_url != transport.base:
-            transport = SvnRaTransport(self.svn_root_url)
-        return SvnRepository(self, transport)
+            transport = transport.clone_root()
+        return SvnRepository(self, transport, self.branch_path)
 
-    def open_workingtree(self):
+    def open_workingtree(self, _unsupported=False,
+            recommend_upgrade=True):
         """See BzrDir.open_workingtree().
 
         Will always raise NotLocalUrl as this 
         BzrDir can not be associated with working trees.
         """
         # Working trees never exist on remote Subversion repositories
-        raise NotLocalUrl(self.root_transport.base)
+        raise NoWorkingTree(self.root_transport.base)
 
     def create_workingtree(self, revision_id=None):
         """See BzrDir.create_workingtree().
@@ -118,35 +125,85 @@ class SvnRemoteAccess(BzrDir):
         """
         raise NotLocalUrl(self.root_transport.base)
 
+    def needs_format_conversion(self, format=None):
+        """See BzrDir.needs_format_conversion()."""
+        # if the format is not the same as the system default,
+        # an upgrade is needed.
+        if format is None:
+            format = BzrDirFormat.get_default_format()
+        return not isinstance(self._format, format.__class__)
+
+    def import_branch(self, source, stop_revision=None):
+        """Create a new branch in this repository, possibly 
+        with the specified history, optionally importing revisions.
+        
+        :param source: Source branch
+        :param stop_revision: Tip of new branch
+        :return: Branch object
+        """
+        from commit import push_new
+        if stop_revision is None:
+            stop_revision = source.last_revision()
+        target_branch_path = self.branch_path.strip("/")
+        repos = self.find_repository()
+        full_branch_url = urlutils.join(repos.transport.base, 
+                                        target_branch_path)
+        if repos.transport.check_path(target_branch_path,
+            repos.transport.get_latest_revnum()) != svn.core.svn_node_none:
+            raise AlreadyBranchError(full_branch_url)
+        push_new(repos, target_branch_path, source, stop_revision)
+        branch = self.open_branch()
+        branch.pull(source, stop_revision=stop_revision)
+        return branch
+
     def create_branch(self):
         """See BzrDir.create_branch()."""
-        repos = self.open_repository()
-        # TODO: Check if there are any revisions in this repository 
-        # yet if it is the top-level one
+        from branch import SvnBranch
+        repos = self.find_repository()
+
+        if self.branch_path != "":
+            # TODO: Set NULL_REVISION in SVN_PROP_BZR_BRANCHING_SCHEME
+            repos.transport.mkdir(self.branch_path.strip("/"))
+        elif repos.transport.get_latest_revnum() > 0:
+            # Bail out if there are already revisions in this repository
+            raise AlreadyBranchError(self.root_transport.base)
         branch = SvnBranch(self.root_transport.base, repos, self.branch_path)
         branch.bzrdir = self
         return branch
 
     def open_branch(self, unsupported=True):
         """See BzrDir.open_branch()."""
-
-        if not self.scheme.is_branch(self.branch_path):
-            raise NotBranchError(path=self.root_transport.base)
-
+        from branch import SvnBranch
         repos = self.find_repository()
         branch = SvnBranch(self.root_transport.base, repos, self.branch_path)
         branch.bzrdir = self
         return branch
 
+    def create_repository(self, shared=False, format=None):
+        """See BzrDir.create_repository."""
+        return self.open_repository()
+
 
 class SvnFormat(BzrDirFormat):
+    """Format for the Subversion smart server."""
     _lock_class = TransportLock
 
+    def __init__(self):
+        super(SvnFormat, self).__init__()
+        from repository import SvnRepositoryFormat
+        self.repository_format = SvnRepositoryFormat()
+
     @classmethod
     def probe_transport(klass, transport):
         format = klass()
 
-        transport = get_svn_ra_transport(transport)
+        try:
+            transport = get_svn_ra_transport(transport)
+        except SubversionException, (_, num):
+            if num in (svn.core.SVN_ERR_RA_ILLEGAL_URL, \
+                       svn.core.SVN_ERR_RA_LOCAL_REPOS_OPEN_FAILED, \
+                       svn.core.SVN_ERR_BAD_URL):
+                raise NotBranchError(path=transport.base)
 
         if isinstance(transport, SvnRaTransport):
             return format
@@ -156,7 +213,7 @@ class SvnFormat(BzrDirFormat):
     def _open(self, transport):
         try: 
             return SvnRemoteAccess(transport, self)
-        except SubversionException, (msg, num):
+        except SubversionException, (_, num):
             if num == svn.core.SVN_ERR_RA_DAV_REQUEST_FAILED:
                 raise NotBranchError(transport.base)
             raise
@@ -175,9 +232,10 @@ class SvnFormat(BzrDirFormat):
                 "non-local transports")
 
         local_path = transport._local_base.rstrip("/")
-        repos = svn.repos.create(local_path, '', '', None, None)
-        return self.open(SvnRaTransport(transport.base), _found=True)
+        svn.repos.create(local_path, '', '', None, None)
+        return self.open(get_svn_ra_transport(transport), _found=True)
 
     def is_supported(self):
         """See BzrDir.is_supported()."""
         return True
+
