@@ -13,13 +13,11 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Cache of the Subversion history log."""
 
-from bzrlib.errors import NoSuchRevision, BzrError, NotBranchError
-from bzrlib.progress import DummyProgress
-from bzrlib.trace import mutter
-from bzrlib.ui import ui_factory
-
-import os
+from bzrlib import urlutils
+from bzrlib.errors import NoSuchRevision
+import bzrlib.ui as ui
 
 from svn.core import SubversionException, Pool
 from transport import SvnRaTransport
@@ -27,12 +25,7 @@ import svn.core
 
 import base64
 
-try:
-    import sqlite3
-except ImportError:
-    from pysqlite2 import dbapi2 as sqlite3
-
-shelves = {}
+from cache import sqlite3
 
 def _escape_commit_message(message):
     """Replace xml-incompatible control characters."""
@@ -57,22 +50,15 @@ def _escape_commit_message(message):
 
 class LogWalker(object):
     """Easy way to access the history of a Subversion repository."""
-    def __init__(self, transport=None, cache_db=None, last_revnum=None):
+    def __init__(self, transport=None, cache_db=None):
         """Create a new instance.
 
         :param transport:   SvnRaTransport to use to access the repository.
         :param cache_db:    Optional sql database connection to use. Doesn't 
                             cache if not set.
-        :param last_revnum: Last known revnum in the repository. Will be 
-                            determined if not specified.
         """
         assert isinstance(transport, SvnRaTransport)
 
-        if last_revnum is None:
-            last_revnum = transport.get_latest_revnum()
-
-        self.last_revnum = last_revnum
-
         self.transport = SvnRaTransport(transport.base)
 
         if cache_db is None:
@@ -85,26 +71,29 @@ class LogWalker(object):
           create unique index if not exists revision_revno on revision (revno);
           create table if not exists changed_path(rev integer, action text, path text, copyfrom_path text, copyfrom_rev integer);
           create index if not exists path_rev on changed_path(rev);
-          create index if not exists path_rev_path on changed_path(rev, path);
+          create unique index if not exists path_rev_path on changed_path(rev, path);
+          create unique index if not exists path_rev_path_action on changed_path(rev, path, action);
         """)
         self.db.commit()
         self.saved_revnum = self.db.execute("SELECT MAX(revno) FROM revision").fetchone()[0]
         if self.saved_revnum is None:
             self.saved_revnum = 0
 
-    def fetch_revisions(self, to_revnum):
+    def fetch_revisions(self, to_revnum=None):
         """Fetch information about all revisions in the remote repository
         until to_revnum.
 
         :param to_revnum: End of range to fetch information for
         """
-        to_revnum = max(self.last_revnum, to_revnum)
+        if to_revnum is None:
+            to_revnum = self.transport.get_latest_revnum()
+        else:
+            to_revnum = max(self.transport.get_latest_revnum(), to_revnum)
 
-        pb = ui_factory.nested_progress_bar()
+        pb = ui.ui_factory.nested_progress_bar()
 
         def rcvr(orig_paths, rev, author, date, message, pool):
             pb.update('fetching svn revision info', rev, to_revnum)
-            paths = {}
             if orig_paths is None:
                 orig_paths = {}
             for p in orig_paths:
@@ -113,7 +102,7 @@ class LogWalker(object):
                     copyfrom_path = copyfrom_path.strip("/")
 
                 self.db.execute(
-                     "insert into changed_path (rev, path, action, copyfrom_path, copyfrom_rev) values (?, ?, ?, ?, ?)", 
+                     "replace into changed_path (rev, path, action, copyfrom_path, copyfrom_rev) values (?, ?, ?, ?, ?)", 
                      (rev, p.strip("/"), orig_paths[p].action, copyfrom_path, orig_paths[p].copyfrom_rev))
 
             if message is not None:
@@ -158,7 +147,7 @@ class LogWalker(object):
 
         path = path.strip("/")
 
-        while revnum > 0:
+        while revnum >= 0:
             revpaths = self.get_revision_paths(revnum, path)
 
             if revpaths != {}:
@@ -175,7 +164,7 @@ class LogWalker(object):
                     revnum = revpaths[path][2]
                     path = revpaths[path][1]
                     continue
-            revnum-=1
+            revnum -= 1
 
     def get_revision_paths(self, revnum, path=None):
         """Obtain dictionary with all the changes in a particular revision.
@@ -198,7 +187,7 @@ class LogWalker(object):
 
         paths = {}
         for p, act, cf, cr in self.db.execute(query):
-            paths[p] = (act, cf, cr)
+            paths[p.encode("utf-8")] = (act, cf, cr)
         return paths
 
     def get_revision_info(self, revnum):
@@ -207,11 +196,15 @@ class LogWalker(object):
         :param revnum: Revision number.
         :returns: Tuple with author, log message and date of the revision.
         """
-        assert revnum >= 1
+        assert revnum >= 0
+        if revnum == 0:
+            return (None, None, None)
         if revnum > self.saved_revnum:
             self.fetch_revisions(revnum)
         (author, message, date) = self.db.execute("select author, message, date from revision where revno="+ str(revnum)).fetchone()
-        return (author, _escape_commit_message(base64.b64decode(message)), date)
+        if message is not None:
+            message = _escape_commit_message(base64.b64decode(message))
+        return (author, message, date)
 
     def find_latest_change(self, path, revnum, recurse=False):
         """Find latest revision that touched path.
@@ -219,6 +212,8 @@ class LogWalker(object):
         :param path: Path to check for changes
         :param revnum: First revision to check
         """
+        assert isinstance(path, basestring)
+        assert isinstance(revnum, int) and revnum >= 0
         if revnum > self.saved_revnum:
             self.fetch_revisions(revnum)
 
@@ -251,25 +246,68 @@ class LogWalker(object):
 
     def find_children(self, path, revnum):
         """Find all children of path in revnum."""
-        # TODO: Find children by walking history, or use 
-        # cache?
+        path = path.strip("/")
+        ft = self.transport.check_path(path, revnum)
+        if ft == svn.core.svn_node_file:
+            return []
+        assert ft == svn.core.svn_node_dir
 
+        class TreeLister(svn.delta.Editor):
+            def __init__(self, base):
+                self.files = []
+                self.base = base
+
+            def set_target_revision(self, revnum):
+                """See Editor.set_target_revision()."""
+                pass
+
+            def open_root(self, revnum, baton):
+                """See Editor.open_root()."""
+                return path
+
+            def add_directory(self, path, parent_baton, copyfrom_path, copyfrom_revnum, pool):
+                """See Editor.add_directory()."""
+                self.files.append(urlutils.join(self.base, path))
+                return path
+
+            def change_dir_prop(self, id, name, value, pool):
+                pass
+
+            def change_file_prop(self, id, name, value, pool):
+                pass
+
+            def add_file(self, path, parent_id, copyfrom_path, copyfrom_revnum, baton):
+                self.files.append(urlutils.join(self.base, path))
+                return path
+
+            def close_dir(self, id):
+                pass
+
+            def close_file(self, path, checksum):
+                pass
+
+            def close_edit(self):
+                pass
+
+            def abort_edit(self):
+                pass
+
+            def apply_textdelta(self, file_id, base_checksum):
+                pass
+        pool = Pool()
+        editor = TreeLister(path)
+        edit, baton = svn.delta.make_editor(editor, pool)
+        old_base = self.transport.base
         try:
-            (dirents, _, _) = self.transport.get_dir(
-                path.lstrip("/").encode('utf8'), revnum, kind=True)
-        except SubversionException, (_, num):
-            if num == svn.core.SVN_ERR_FS_NOT_DIRECTORY:
-                return
-            raise
-
-        for p in dirents:
-            yield os.path.join(path, p)
-            # This needs to be != svn.core.svn_node_file because 
-            # some ra backends seem to return negative values for .kind.
-            # however, dirents[p].node seems to contain semi-random 
-            # values.
-            for c in self.find_children(os.path.join(path, p), revnum):
-                yield c
+            root_repos = self.transport.get_repos_root()
+            self.transport.reparent(urlutils.join(root_repos, path))
+            reporter = self.transport.do_update(
+                            revnum,  True, edit, baton, pool)
+            reporter.set_path("", revnum, True, None, pool)
+            reporter.finish_report(pool)
+        finally:
+            self.transport.reparent(old_base)
+        return editor.files
 
     def get_previous(self, path, revnum):
         """Return path,revnum pair specified pair was derived from.
