@@ -14,17 +14,20 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import svn.repos
+"""Tests for the bzr-svn plugin."""
+
 import os
 import bzrlib
 from bzrlib import osutils
 from bzrlib.bzrdir import BzrDir
-from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
 from bzrlib.trace import mutter
+from bzrlib.workingtree import WorkingTree
 
 RENAMES = False
 
-import svn.ra, svn.repos, svn.wc
+import svn.repos, svn.wc
+from errors import NoCheckoutSupport
 
 class TestCaseWithSubversionRepository(TestCaseInTempDir):
     """A test case that provides the ability to build Subversion 
@@ -47,7 +50,7 @@ class TestCaseWithSubversionRepository(TestCaseInTempDir):
         abspath = os.path.join(self.test_dir, relpath)
         repos_url = "file://%s" % abspath
 
-        repos = svn.repos.create(abspath, '', '', None, None)
+        svn.repos.create(abspath, '', '', None, None)
 
         revprop_hook = os.path.join(abspath, "hooks", "pre-revprop-change")
 
@@ -76,7 +79,11 @@ class TestCaseWithSubversionRepository(TestCaseInTempDir):
 
         repos_url = self.make_repository(repos_path)
 
-        return self.open_local_bzrdir(repos_url, relpath)
+        try:
+            return self.open_local_bzrdir(repos_url, relpath)
+        except NoCheckoutSupport:
+            raise TestSkipped('No Checkout Support')
+
 
     def make_checkout(self, repos_url, relpath):
         rev = svn.core.svn_opt_revision_t()
@@ -85,6 +92,35 @@ class TestCaseWithSubversionRepository(TestCaseInTempDir):
         svn.client.checkout2(repos_url, relpath, 
                 rev, rev, True, False, self.client_ctx)
 
+    @staticmethod
+    def create_checkout(branch, path, revision_id=None, lightweight=False):
+        try:
+            return branch.create_checkout(path, revision_id=revision_id,
+                                          lightweight=lightweight)
+        except NoCheckoutSupport:
+            raise TestSkipped('No Checkout Support')
+
+    @staticmethod
+    def open_checkout(url):
+        try:
+            return WorkingTree.open(url)
+        except NoCheckoutSupport:
+           raise TestSkipped('No Checkout Support')
+
+    @staticmethod
+    def open_checkout_bzrdir(url):
+        try:
+            return BzrDir.open(url)
+        except NoCheckoutSupport:
+           raise TestSkipped('No Checkout Support')
+
+    @staticmethod
+    def create_branch_convenience(url):
+        try:
+            return BzrDir.create_branch_convenience(url)
+        except NoCheckoutSupport:
+           raise TestSkipped('No Checkout Support')
+
     def client_set_prop(self, path, name, value):
         if value is None:
             value = ""
@@ -121,6 +157,7 @@ class TestCaseWithSubversionRepository(TestCaseInTempDir):
         os.chdir(dir)
         info = svn.client.commit3(["."], recursive, False, self.client_ctx)
         os.chdir(olddir)
+        assert info is not None
         return (info.revision, info.date, info.author)
 
     def client_add(self, relpath, recursive=True):
@@ -130,6 +167,27 @@ class TestCaseWithSubversionRepository(TestCaseInTempDir):
         """
         svn.client.add3(relpath, recursive, False, False, self.client_ctx)
 
+    def revnum_to_opt_rev(self, revnum):
+        rev = svn.core.svn_opt_revision_t()
+        if revnum is None:
+            rev.kind = svn.core.svn_opt_revision_head
+        else:
+            rev.kind = svn.core.svn_opt_revision_number
+            rev.value.number = revnum
+        return rev
+
+    def client_log(self, path, start_revnum=None, stop_revnum=None):
+        ret = {}
+        def rcvr(orig_paths, rev, author, date, message, pool):
+            ret[rev] = (orig_paths, author, date, message)
+        svn.client.log([path], self.revnum_to_opt_rev(start_revnum),
+                       self.revnum_to_opt_rev(stop_revnum),
+                       True,
+                       True,
+                       rcvr,
+                       self.client_ctx)
+        return ret
+
     def client_delete(self, relpath):
         """Remove specified files from working copy.
 
@@ -191,17 +249,6 @@ class TestCaseWithSubversionRepository(TestCaseInTempDir):
         self.make_checkout(repos_url, clientpath)
         return repos_url
 
-    def make_ra(self, relpath):
-        """Create a repository and a ra connection to it. 
-        
-        :param relpath: Path to create repository at.
-        :return: The ra connection.
-        """
-
-        repos_url = self.make_repository(relpath)
-
-        return svn.ra.open2(repos_url, svn.ra.callbacks2_t(), None, None)
-
     def dumpfile(self, repos):
         """Create a dumpfile for the specified repository.
 
@@ -220,7 +267,7 @@ class TestCaseWithSubversionRepository(TestCaseInTempDir):
 
 
 def test_suite():
-    from unittest import TestSuite, TestLoader
+    from unittest import TestSuite
     
     from bzrlib.tests import TestUtil
 
@@ -233,16 +280,22 @@ def test_suite():
             'test_branchprops', 
             'test_checkout',
             'test_commit',
+            'test_config',
             'test_convert',
+            'test_errors',
+            'test_fetch',
             'test_fileids', 
             'test_logwalker',
+            'test_push',
             'test_radir',
             'test_repos', 
+            'test_revids',
             'test_scheme', 
             'test_transport',
             'test_tree',
             'test_upgrade',
-            'test_workingtree']
+            'test_workingtree',
+            'test_blackbox']
     suite.addTest(loader.loadTestsFromModuleNames(["%s.%s" % (__name__, i) for i in testmod_names]))
 
     return suite
