@@ -14,12 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+"""Branch property access tests."""
+
 from bzrlib.errors import NoSuchRevision
 
 from tests import TestCaseWithSubversionRepository
 from branchprops import BranchPropertyList
 from logwalker import LogWalker
-from scheme import NoBranchingScheme
 from transport import SvnRaTransport
 
 try:
@@ -42,6 +43,17 @@ class TestBranchProps(TestCaseWithSubversionRepository):
         bp = BranchPropertyList(logwalk, self.db)
         self.assertEqual("data", bp.get_property("", 1, "myprop"))
 
+    def test_get_property_multiple(self):
+        repos_url = self.make_client('d', 'dc')
+        self.client_set_prop("dc", "aprop", "foo")
+        self.client_set_prop("dc", "myprop", "data")
+        self.client_commit("dc", "My Message")
+
+        logwalk = LogWalker(transport=SvnRaTransport(repos_url))
+
+        bp = BranchPropertyList(logwalk, self.db)
+        self.assertEqual("data", bp.get_property("", 1, "myprop"))
+
     def test_get_property_norev(self):
         repos_url = self.make_client('d', 'dc')
         self.client_set_prop("dc", "myprop", "data")
@@ -103,6 +115,20 @@ class TestBranchProps(TestCaseWithSubversionRepository):
         bp = BranchPropertyList(logwalk, self.db)
         self.assertEqual("data2\n", bp.get_property_diff("", 2, "myprop"))
 
+    def test_touches_property(self):
+        repos_url = self.make_client('d', 'dc')
+        self.client_set_prop("dc", "myprop", "data\n")
+        self.client_commit("dc", "My Message")
+        self.client_set_prop("dc", "myprop", "data\ndata2\n")
+        self.client_commit("dc", "My Message")
+
+        logwalk = LogWalker(transport=SvnRaTransport(repos_url))
+
+        bp = BranchPropertyList(logwalk, self.db)
+        self.assertTrue(bp.touches_property("", 2, "myprop"))
+        self.assertTrue(bp.touches_property("", 1, "myprop"))
+        self.assertFalse(bp.touches_property("", 1, "nonexistant-property"))
+
     def test_get_property_diff_ignore_origchange(self):
         repos_url = self.make_client('d', 'dc')
         self.client_set_prop("dc", "myprop", "foodata\n")
