@@ -14,15 +14,18 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+"""Full repository conversion tests."""
+
 from bzrlib.branch import Branch
-from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NotBranchError
+from bzrlib.bzrdir import BzrDir, format_registry
+from bzrlib.errors import NotBranchError, NoSuchFile, IncompatibleRepositories
 from bzrlib.repository import Repository
-from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.trace import mutter
 
 import os
 from convert import convert_repository, NotDumpFile, load_dumpfile
+from format import get_rich_root_format
 from repository import generate_svn_revision_id
 from scheme import TrunkBranchingScheme, NoBranchingScheme
 from tests import TestCaseWithSubversionRepository
@@ -55,7 +58,7 @@ PROPS-END
 
     def test_loaddumpfile_invalid(self):
         dumpfile = os.path.join(self.test_dir, "dumpfile")
-        open(dumpfile, 'w').write("""FooBar""")
+        open(dumpfile, 'w').write("""FooBar\n""")
         self.assertRaises(NotDumpFile, load_dumpfile, dumpfile, "d")
 
 
@@ -70,26 +73,36 @@ class TestConversion(TestCaseWithSubversionRepository):
         self.build_tree({'dc/trunk/file': 'otherdata'})
         self.client_commit("dc", "change")
 
+    def test_sets_parent_urls(self):
+        convert_repository(Repository.open(self.repos_url), "e", 
+                           TrunkBranchingScheme(), 
+                           all=False, create_shared_repo=True)
+        self.assertEquals(self.repos_url+"/trunk", 
+                Branch.open("e/trunk").get_parent())
+        self.assertEquals(self.repos_url+"/branches/abranch", 
+                Branch.open("e/branches/abranch").get_parent())
+
     def test_fetch_alive(self):
         self.build_tree({'dc/branches/somebranch/somefile': 'data'})
         self.client_add("dc/branches/somebranch")
         self.client_commit("dc", "add a branch")
         self.client_delete("dc/branches/somebranch")
         self.client_commit("dc", "remove branch")
-        convert_repository(self.repos_url, "e", TrunkBranchingScheme(), 
-                           all=False, create_shared_repo=True)
         oldrepos = Repository.open(self.repos_url)
+        convert_repository(oldrepos, "e", 
+                           TrunkBranchingScheme(), 
+                           all=False, create_shared_repo=True)
         newrepos = Repository.open("e")
-        self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch")))
+        self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch", "trunk0")))
 
     def test_fetch_filebranch(self):
         self.build_tree({'dc/branches/somebranch': 'data'})
         self.client_add("dc/branches/somebranch")
         self.client_commit("dc", "add a branch")
-        convert_repository(self.repos_url, "e", TrunkBranchingScheme())
         oldrepos = Repository.open(self.repos_url)
+        convert_repository(oldrepos, "e", TrunkBranchingScheme())
         newrepos = Repository.open("e")
-        self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch")))
+        self.assertFalse(newrepos.has_revision(oldrepos.generate_revision_id(2, "branches/somebranch", "trunk0")))
 
     def test_fetch_dead(self):
         self.build_tree({'dc/branches/somebranch/somefile': 'data'})
@@ -97,81 +110,124 @@ class TestConversion(TestCaseWithSubversionRepository):
         self.client_commit("dc", "add a branch")
         self.client_delete("dc/branches/somebranch")
         self.client_commit("dc", "remove branch")
-        convert_repository(self.repos_url, "e", TrunkBranchingScheme(), 
-                           all=True, create_shared_repo=True)
         oldrepos = Repository.open(self.repos_url)
+        convert_repository(oldrepos, "e", TrunkBranchingScheme(), 
+                           all=True, create_shared_repo=True)
         newrepos = Repository.open("e")
-        mutter('q: %r' % newrepos.all_revision_ids())
-        self.assertTrue(newrepos.has_revision(oldrepos.generate_revision_id(3, "branches/somebranch")))
+        self.assertTrue(newrepos.has_revision(
+            oldrepos.generate_revision_id(3, "branches/somebranch", "trunk0")))
+
+    def test_fetch_filter(self):
+        self.build_tree({'dc/branches/somebranch/somefile': 'data',
+                         'dc/branches/anotherbranch/somefile': 'data'})
+        self.client_add("dc/branches/somebranch")
+        self.client_commit("dc", "add a branch")
+        self.client_add("dc/branches/anotherbranch")
+        self.client_commit("dc", "add another branch")
+        oldrepos = Repository.open(self.repos_url)
+        convert_repository(oldrepos, "e", TrunkBranchingScheme(), 
+            create_shared_repo=True,
+            filter_branch=lambda (bp, revnum, exists): bp.endswith("somebranch"))
+        newrepos = Repository.open("e")
+        self.assertTrue(os.path.exists("e/branches/somebranch"))
+        self.assertFalse(os.path.exists("e/branches/anotherbranch"))
 
     def test_shared_import_continue(self):
-        BzrDir.create_repository("e", shared=True)
+        BzrDir.create_repository("e", shared=True, 
+                format=get_rich_root_format())
 
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), True)
 
         self.assertTrue(Repository.open("e").is_shared())
 
+    def test_shared_import_continue_remove(self):
+        convert_repository(Repository.open(self.repos_url), "e", 
+                TrunkBranchingScheme(), create_shared_repo=True)
+        self.client_update("dc")
+        self.client_delete("dc/trunk")
+        self.client_commit("dc", "blafoo")
+        self.build_tree({'dc/trunk/file': 'otherdata'})
+        self.client_add("dc/trunk")
+        self.client_commit("dc", "change")
+        convert_repository(Repository.open(self.repos_url), "e", 
+                           TrunkBranchingScheme(), create_shared_repo=True)
+
     def test_shared_import_continue_with_wt(self):
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), working_trees=True)
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), working_trees=True)
 
     def test_shared_import_nonescheme_empty(self):
-        BzrDir.create_repository("e", shared=True)
+        BzrDir.create_repository("e", shared=True, format=get_rich_root_format())
 
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 NoBranchingScheme(), True)
 
     def test_shared_import_with_wt(self):
-        BzrDir.create_repository("e", shared=True)
+        BzrDir.create_repository("e", shared=True, 
+                format=get_rich_root_format())
 
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), True, True)
 
         self.assertTrue(os.path.isfile(os.path.join(
                         self.test_dir, "e", "trunk", "file")))
 
     def test_shared_import_without_wt(self):
-        BzrDir.create_repository("e", shared=True)
+        BzrDir.create_repository("e", shared=True, 
+                format=get_rich_root_format())
 
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), True, False)
 
         self.assertFalse(os.path.isfile(os.path.join(
                         self.test_dir, "e", "trunk", "file")))
 
+    def test_shared_import_old_repos_fails(self):
+        BzrDir.create_repository("e", shared=True, 
+                format=format_registry.make_bzrdir('knit'))
+
+        self.assertRaises(IncompatibleRepositories, 
+            lambda: convert_repository(Repository.open(self.repos_url), "e", 
+                TrunkBranchingScheme(), True, False))
+
     def test_shared_import_continue_branch(self):
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), True)
 
         self.build_tree({'dc/trunk/file': 'foodata'})
         self.client_commit("dc", "msg")
 
-        self.assertEqual(Repository.open(self.repos_url).generate_revision_id(2, "trunk"), Branch.open("e/trunk").last_revision())
+        self.assertEqual(
+                Repository.open(self.repos_url).generate_revision_id(2, "trunk", "trunk0"), 
+                Branch.open("e/trunk").last_revision())
 
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), True)
 
-        self.assertEqual(Repository.open(self.repos_url).generate_revision_id(3, "trunk"), 
+        self.assertEqual(Repository.open(self.repos_url).generate_revision_id(3, "trunk", "trunk0"), 
                         Branch.open("e/trunk").last_revision())
 
  
     def test_shared_import(self):
-        convert_repository("svn+"+self.repos_url, "e", 
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
                 TrunkBranchingScheme(), True)
 
         self.assertTrue(Repository.open("e").is_shared())
     
     def test_simple(self):
-        convert_repository("svn+"+self.repos_url, os.path.join(self.test_dir, "e"), TrunkBranchingScheme())
+        convert_repository(Repository.open("svn+"+self.repos_url), os.path.join(self.test_dir, "e"), TrunkBranchingScheme())
         self.assertTrue(os.path.isdir(os.path.join(self.test_dir, "e", "trunk")))
         self.assertTrue(os.path.isdir(os.path.join(self.test_dir, "e", "branches", "abranch")))
 
+    def test_convert_to_nonexistant(self):
+        self.assertRaises(NoSuchFile, convert_repository,Repository.open("svn+"+self.repos_url), os.path.join(self.test_dir, "e", "foo", "bar"), TrunkBranchingScheme())
+
     def test_notshared_import(self):
-        convert_repository("svn+"+self.repos_url, "e", TrunkBranchingScheme(), 
-                           False)
+        convert_repository(Repository.open("svn+"+self.repos_url), "e", 
+                           TrunkBranchingScheme(), False)
 
         self.assertRaises(NotBranchError, Repository.open, "e")
 
@@ -194,11 +250,16 @@ V 27
 PROPS-END
 """)
         branch_path = os.path.join(self.test_dir, "f")
-        convert_repository(dumpfile, branch_path, NoBranchingScheme())
+        repos = self.load_dumpfile(dumpfile, 'g')
+        convert_repository(repos, branch_path, NoBranchingScheme())
         branch = Repository.open(branch_path)
-        self.assertEqual([], branch.all_revision_ids())
+        self.assertEqual(['svn-v3-none:6987ef2d-cd6b-461f-9991-6f1abef3bd59::0'], branch.all_revision_ids())
         Branch.open(branch_path)
 
+    def load_dumpfile(self, dumpfile, target_path):
+        load_dumpfile(dumpfile, target_path)
+        return Repository.open(target_path)
+
     def test_dumpfile_open_empty_trunk(self):
         dumpfile = os.path.join(self.test_dir, "dumpfile")
         open(dumpfile, 'w').write(
@@ -217,7 +278,8 @@ V 27
 PROPS-END
 """)
         branch_path = os.path.join(self.test_dir, "f")
-        convert_repository(dumpfile, branch_path, TrunkBranchingScheme())
+        repos = self.load_dumpfile(dumpfile, 'g')
+        convert_repository(repos, branch_path, TrunkBranchingScheme())
         repository = Repository.open(branch_path)
         self.assertEqual([], repository.all_revision_ids())
         self.assertRaises(NotBranchError, Branch.open, branch_path)
@@ -279,8 +341,10 @@ data
 
 
 """)
-        convert_repository(filename, os.path.join(self.test_dir, "e"), 
+        repos = self.load_dumpfile(filename, 'g')
+        convert_repository(repos, os.path.join(self.test_dir, "e"), 
                            TrunkBranchingScheme())
         branch = Branch.open(os.path.join(self.test_dir, "e", "trunk"))
         self.assertEqual("file://%s/e/trunk" % self.test_dir, branch.base.rstrip("/"))
-        self.assertEqual(generate_svn_revision_id("6987ef2d-cd6b-461f-9991-6f1abef3bd59", 1, 'trunk'), branch.last_revision())
+        self.assertEqual(generate_svn_revision_id("6987ef2d-cd6b-461f-9991-6f1abef3bd59", 1, 'trunk', "trunk0"), branch.last_revision())
+
