@@ -14,21 +14,30 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+"""File id tests."""
+
 from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NoSuchRevision
-from bzrlib.inventory import Inventory, ROOT_ID
 from bzrlib.repository import Repository
 from bzrlib.trace import mutter
-from bzrlib.tests import TestSkipped, TestCase
+from bzrlib.tests import TestCase
 
 import sha
 
-import format
 from fileids import SimpleFileIdMap, generate_file_id, generate_svn_file_id
-from repository import MAPPING_VERSION
+from repository import generate_svn_revision_id
+from revids import parse_svn_revision_id
 from scheme import TrunkBranchingScheme
 from tests import TestCaseWithSubversionRepository, RENAMES
 
+class MockRepo:
+    def __init__(self, uuid="uuid"):
+        self.uuid = uuid
+
+    def lookup_revision_id(self, revid):
+        ret = parse_svn_revision_id(revid)
+        return ret[1], ret[2], ret[3]
+
+
 class TestComplexFileids(TestCaseWithSubversionRepository):
     # branchtagcopy.dump
     # changeaftercp.dump
@@ -52,9 +61,9 @@ class TestComplexFileids(TestCaseWithSubversionRepository):
         repository = Repository.open("svn+"+repos_url)
 
         inv1 = repository.get_inventory(
-                repository.generate_revision_id(1, ""))
+                repository.generate_revision_id(1, "", "none"))
         inv2 = repository.get_inventory(
-                repository.generate_revision_id(2, ""))
+                repository.generate_revision_id(2, "", "none"))
         mutter('inv1: %r' % inv1.entries())
         mutter('inv2: %r' % inv2.entries())
         if RENAMES:
@@ -79,9 +88,9 @@ class TestComplexFileids(TestCaseWithSubversionRepository):
         repository = bzrdir.find_repository()
 
         inv1 = repository.get_inventory(
-                repository.generate_revision_id(1, ""))
+                repository.generate_revision_id(1, "", "none"))
         inv2 = repository.get_inventory(
-                repository.generate_revision_id(2, ""))
+                repository.generate_revision_id(2, "", "none"))
         self.assertNotEqual(inv1.path2id("foo"), inv2.path2id("bar"))
         self.assertNotEqual(inv1.path2id("foo"), inv2.path2id("blie"))
         self.assertIs(None, inv1.path2id("bar"))
@@ -99,9 +108,9 @@ class TestComplexFileids(TestCaseWithSubversionRepository):
         repository = bzrdir.find_repository()
 
         inv1 = repository.get_inventory(
-                repository.generate_revision_id(1, ""))
+                repository.generate_revision_id(1, "", "none"))
         inv2 = repository.get_inventory(
-                repository.generate_revision_id(2, ""))
+                repository.generate_revision_id(2, "", "none"))
         self.assertNotEqual(None, inv1.path2id("foo"))
         self.assertIs(None, inv2.path2id("foo"))
 
@@ -119,12 +128,13 @@ class TestComplexFileids(TestCaseWithSubversionRepository):
         repository = bzrdir.find_repository()
 
         inv1 = repository.get_inventory(
-                repository.generate_revision_id(1, ""))
+                repository.generate_revision_id(1, "", "none"))
         inv2 = repository.get_inventory(
-                repository.generate_revision_id(2, ""))
+                repository.generate_revision_id(2, "", "none"))
         self.assertNotEqual(inv1.path2id("foo"), inv2.path2id("foo"))
 
     def test_copy_branch(self):
+        scheme = TrunkBranchingScheme()
         repos_url = self.make_client('d', 'dc')
         self.build_tree({'dc/trunk/dir/file': "data", 'dc/branches': None})
         self.client_add("dc/trunk")
@@ -137,47 +147,55 @@ class TestComplexFileids(TestCaseWithSubversionRepository):
         repository = bzrdir.find_repository()
 
         inv1 = repository.get_inventory(
-                repository.generate_revision_id(1, "trunk"))
+                repository.generate_revision_id(1, "trunk", "trunk0"))
         inv2 = repository.get_inventory(
-                repository.generate_revision_id(2, "branches/mybranch"))
+                repository.generate_revision_id(2, "branches/mybranch", "trunk0"))
         self.assertEqual(inv1.path2id("dir"), inv2.path2id("dir"))
         self.assertEqual(inv1.path2id("dir/file"), inv2.path2id("dir/file"))
 
         fileid, revid = repository.get_fileid_map(2, 
-                            "branches/mybranch")["dir/file"]
+                            "branches/mybranch", scheme)["dir/file"]
         self.assertEqual(fileid, inv1.path2id("dir/file"))
-        self.assertEqual(repository.generate_revision_id(1, "trunk"), revid)
+        self.assertEqual(repository.generate_revision_id(1, "trunk", "trunk0"), revid)
 
-def sha1(str):
-    return sha.new(str).hexdigest()
+def sha1(text):
+    return sha.new(text).hexdigest()
 
 class TestFileIdGenerator(TestCase):
+    def setUp(self):
+        super(TestCase, self).setUp()
+        self.repos = MockRepo()
+
     def test_generate_file_id_root(self):
-        self.assertEqual(ROOT_ID, generate_file_id("svn-v2:2@uuid-bp", ""))
+        self.assertEqual("2@uuid:bp:", generate_file_id(self.repos, generate_svn_revision_id("uuid", 2, "bp", "bla"), ""))
 
     def test_generate_file_id_path(self):
-        self.assertEqual("svn-v2:2@uuid-bp-mypath", 
-                         generate_file_id("svn-v2:2@uuid-bp", "mypath"))
+        self.assertEqual("2@uuid:bp:mypath", 
+                generate_file_id(self.repos, generate_svn_revision_id("uuid", 2, "bp", "bla"), "mypath"))
 
     def test_generate_file_id_long(self):
         dir = "this/is/a" + ("/very"*40) + "/long/path/"
-        self.assertEqual("svn-v2:2@uuid-bp-" + sha1(dir) + "-filename", 
-                         generate_file_id("svn-v2:2@uuid-bp", dir+"filename"))
+        self.assertEqual("2@uuid:bp;" + sha1(dir+"filename"), 
+                generate_file_id(self.repos, generate_svn_revision_id("uuid", 2, "bp", "bla"), dir+"filename"))
+
+    def test_generate_revid_special_char_ascii(self):
+        self.assertEqual("2@uuid:bp:mypath%2C%8A", 
+                generate_file_id(self.repos, generate_svn_revision_id("uuid", 2, "bp", "bla"), "mypath\x2c\x8a"))
 
     def test_generate_file_id_special_char(self):
-        self.assertEqual(u"svn-v2:2@uuid-bp-mypath\x2c\x8a".encode("utf-8"), 
-                         generate_file_id("svn-v2:2@uuid-bp", u"mypath\x2c\x8a"))
+        self.assertEqual("2@uuid:bp:mypath%2C%C2%8A",
+                         generate_file_id(self.repos, generate_svn_revision_id("uuid", 2, "bp","bla"), u"mypath\x2c\x8a"))
 
     def test_generate_svn_file_id(self):
-        self.assertEqual("svn-v2:2@uuid-bp-path", 
+        self.assertEqual("2@uuid:bp:path", 
                 generate_svn_file_id("uuid", 2, "bp", "path"))
 
     def test_generate_svn_file_id_nordic(self):
-        self.assertEqual("svn-v2:2@uuid-bp-\xc3\xa6\xc3\xb8\xc3\xa5", 
+        self.assertEqual("2@uuid:bp:%C3%A6%C3%B8%C3%A5", 
                 generate_svn_file_id("uuid", 2, "bp", u"\xe6\xf8\xe5"))
 
     def test_generate_svn_file_id_nordic_branch(self):
-        self.assertEqual("svn-v2:2@uuid-\xc3\xa6-\xc3\xa6\xc3\xb8\xc3\xa5", 
+        self.assertEqual("2@uuid:%C3%A6:%C3%A6%C3%B8%C3%A5", 
                 generate_svn_file_id("uuid", 2, u"\xe6", u"\xe6\xf8\xe5"))
 
 
@@ -187,25 +205,25 @@ class TestFileMapping(TestCase):
         revids = mappings.keys()
         revids.sort()
         for r in revids:
-             def new_file_id(x):
-                 if renames.has_key(r) and renames[r].has_key(x):
-                     return renames[r][x]
-                 return generate_file_id(r, x)
-             revmap = SimpleFileIdMap._apply_changes(new_file_id, mappings[r], find_children)
-             map.update(dict([(x,(revmap[x],r)) for x in revmap]))
+            def new_file_id(x):
+                if renames.has_key(r) and renames[r].has_key(x):
+                    return renames[r][x]
+                return generate_file_id(MockRepo(), r, x)
+            revmap = SimpleFileIdMap._apply_changes(new_file_id, mappings[r], find_children)
+            map.update(dict([(x, (revmap[x],r)) for x in revmap]))
         return map
 
     def test_simple(self):
-        map = self.apply_mappings({"svn-v%d:1@uuid-" % MAPPING_VERSION: {"foo": ('A', None, None)}})
-        self.assertEqual({ 'foo': ("svn-v%d:1@uuid--foo" % MAPPING_VERSION, 
-                                       "svn-v%d:1@uuid-" % MAPPING_VERSION)
+        map = self.apply_mappings({generate_svn_revision_id("uuid", 1, "", "bla"): {"foo": ('A', None, None)}})
+        self.assertEqual({ 'foo': ("1@uuid::foo",
+                                       generate_svn_revision_id("uuid", 1, "", "bla"))
                          }, map)
 
     def test_simple_add(self):
-        map = self.apply_mappings({"svn-v%d:1@uuid-" % MAPPING_VERSION: {"": ('A', None, None), "foo": ('A', None, None)}})
-        self.assertEqual({'': ('TREE_ROOT', "svn-v%d:1@uuid-" % MAPPING_VERSION), 
-                               'foo': ("svn-v%d:1@uuid--foo" % MAPPING_VERSION, 
-                                       "svn-v%d:1@uuid-" % MAPPING_VERSION)
+        map = self.apply_mappings({generate_svn_revision_id("uuid", 1, "", "bla"): {"": ('A', None, None), "foo": ('A', None, None)}})
+        self.assertEqual({'': ('1@uuid::', generate_svn_revision_id("uuid", 1, "", "bla")),
+            'foo': ("1@uuid::foo", 
+                                       generate_svn_revision_id("uuid", 1, "", "bla"))
                          }, map)
 
     def test_copy(self):
@@ -214,11 +232,11 @@ class TestFileMapping(TestCase):
                 yield "foo/blie"
                 yield "foo/bla"
         map = self.apply_mappings(
-                {"svn-v%d:1@uuid-" % MAPPING_VERSION: {
+                {generate_svn_revision_id("uuid", 1, "", "bla"): {
                                    "foo": ('A', None, None), 
                                    "foo/blie": ('A', None, None),
                                    "foo/bla": ('A', None, None)},
-                "svn-v%d:2@uuid-" % MAPPING_VERSION: {
+                generate_svn_revision_id("uuid", 2, "", "bla"): {
                                    "foob": ('A', 'foo', 1), 
                                    "foob/bla": ('M', None, None)}
                 }, find_children)
@@ -227,37 +245,39 @@ class TestFileMapping(TestCase):
 
     def test_touchparent(self):
         map = self.apply_mappings(
-                {("svn-v%d:1@uuid-" % MAPPING_VERSION): {
+                {generate_svn_revision_id("uuid", 1, "", "bla"): {
                                    "foo": ('A', None, None), 
                                    "foo/bla": ('A', None, None)},
-                 ("svn-v%d:2@uuid-" % MAPPING_VERSION): {
+                 generate_svn_revision_id("uuid", 2, "", "bla"): {
                                    "foo/bla": ('M', None, None)}
                 })
-        self.assertEqual("svn-v%d:1@uuid-" % MAPPING_VERSION, map["foo"][1])
-        self.assertEqual("svn-v%d:1@uuid-" % MAPPING_VERSION, map["foo/bla"][1])
+        self.assertEqual(generate_svn_revision_id("uuid", 1, "", "bla"), 
+                         map["foo"][1])
+        self.assertEqual(generate_svn_revision_id("uuid", 1, "", "bla"), 
+                         map["foo/bla"][1])
 
     def test_usemap(self):
         map = self.apply_mappings(
-                {("svn-v%d:1@uuid-" % MAPPING_VERSION): {
+                {generate_svn_revision_id("uuid", 1, "", "bla"): {
                                    "foo": ('A', None, None), 
                                    "foo/bla": ('A', None, None)},
-                 ("svn-v%d:2@uuid-" % MAPPING_VERSION): {
+                 generate_svn_revision_id("uuid", 2, "", "bla"): {
                                    "foo/bla": ('M', None, None)}
                  }, 
-                renames={("svn-v%d:1@uuid-" % MAPPING_VERSION): {"foo": "myid"}})
+                renames={generate_svn_revision_id("uuid", 1, "", "bla"): {"foo": "myid"}})
         self.assertEqual("myid", map["foo"][0])
 
     def test_usemap_later(self):
         map = self.apply_mappings(
-                {("svn-v%d:1@uuid-" % MAPPING_VERSION): {
+                {generate_svn_revision_id("uuid", 1, "", "bla"): {
                                    "foo": ('A', None, None), 
                                    "foo/bla": ('A', None, None)},
-                 ("svn-v%d:2@uuid-" % MAPPING_VERSION): {
+                 generate_svn_revision_id("uuid", 2, "", "bla"): {
                                    "foo/bla": ('M', None, None)}
                  }, 
-                renames={("svn-v%d:2@uuid-" % MAPPING_VERSION): {"foo": "myid"}})
-        self.assertEqual("svn-v%d:1@uuid--foo" % MAPPING_VERSION, map["foo"][0])
-        self.assertEqual("svn-v%d:1@uuid-" % MAPPING_VERSION, map["foo"][1])
+                renames={generate_svn_revision_id("uuid", 2, "", "bla"): {"foo": "myid"}})
+        self.assertEqual("1@uuid::foo", map["foo"][0])
+        self.assertEqual(generate_svn_revision_id("uuid", 1, "", "bla"), map["foo"][1])
 
 class GetMapTests(TestCaseWithSubversionRepository):
     def setUp(self):
@@ -266,29 +286,30 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.repos = Repository.open(self.repos_url)
 
     def test_empty(self):
-        self.assertEqual({"": (ROOT_ID, None)}, 
-                         self.repos.get_fileid_map(0, ""))
+        scheme = TrunkBranchingScheme()
+        self.assertEqual({"": (generate_svn_file_id(self.repos.uuid, 0, "", ""), self.repos.generate_revision_id(0, "", "trunk0"))}, 
+                         self.repos.get_fileid_map(0, "", scheme))
 
     def test_empty_trunk(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        scheme = TrunkBranchingScheme()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
         self.client_commit("dc", "Msg")
-        self.assertEqual({"": (ROOT_ID, "svn-v%d:1@%s-trunk" % (MAPPING_VERSION, 
-                               self.repos.uuid))}, self.repos.get_fileid_map(1, "trunk"))
+        self.assertEqual({"": (generate_svn_file_id(self.repos.uuid, 1, "trunk", ""), self.repos.generate_revision_id(1, "trunk", "trunk0"))}, 
+                self.repos.get_fileid_map(1, "trunk", scheme))
 
     def test_change_parent(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        scheme = TrunkBranchingScheme()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
         self.client_commit("dc", "Msg")
         self.build_tree({"dc/trunk/file": 'data'})
         self.client_add("dc/trunk/file")
         self.client_commit("dc", "Msg")
-        self.assertEqual({"": (ROOT_ID, "svn-v%d:2@%s-trunk" % (MAPPING_VERSION, self.repos.uuid)), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), "svn-v%d:2@%s-trunk" % (MAPPING_VERSION, self.repos.uuid))}, self.repos.get_fileid_map(2, "trunk"))
+        self.assertEqual({"": (generate_svn_file_id(self.repos.uuid, 1, "trunk", ""), self.repos.generate_revision_id(2, "trunk", "trunk0")), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), self.repos.generate_revision_id(2, "trunk", "trunk0"))}, self.repos.get_fileid_map(2, "trunk", scheme))
 
     def test_change_updates(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        scheme = TrunkBranchingScheme()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
         self.client_commit("dc", "Msg")
@@ -297,23 +318,26 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.client_commit("dc", "Msg")
         self.build_tree({"dc/trunk/file": 'otherdata'})
         self.client_commit("dc", "Msg")
-        self.assertEqual({"": (ROOT_ID, self.repos.generate_revision_id(3, "trunk")), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), self.repos.generate_revision_id(3, "trunk"))}, self.repos.get_fileid_map(3, "trunk"))
+        self.assertEqual({"": (generate_svn_file_id(self.repos.uuid, 1, "trunk", ""), self.repos.generate_revision_id(3, "trunk", "trunk0")), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), self.repos.generate_revision_id(3, "trunk", "trunk0"))}, self.repos.get_fileid_map(3, "trunk", scheme))
 
     def test_sibling_unrelated(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        scheme = TrunkBranchingScheme()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
         self.client_commit("dc", "Msg")
+        self.client_update("dc")
         self.build_tree({"dc/trunk/file": 'data', 'dc/trunk/bar': 'data2'})
         self.client_add("dc/trunk/file")
         self.client_add("dc/trunk/bar")
         self.client_commit("dc", "Msg")
+        self.client_update("dc")
         self.build_tree({"dc/trunk/file": 'otherdata'})
         self.client_commit("dc", "Msg")
-        self.assertEqual({"": (ROOT_ID, self.repos.generate_revision_id(3, "trunk")), "bar": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "bar"), "svn-v%d:2@%s-trunk" % (MAPPING_VERSION, self.repos.uuid)), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), self.repos.generate_revision_id(3, "trunk"))}, self.repos.get_fileid_map(3, "trunk"))
+        self.client_update("dc")
+        self.assertEqual({"": (generate_svn_file_id(self.repos.uuid, 1, "trunk", ""), self.repos.generate_revision_id(3, "trunk", "trunk0")), "bar": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "bar"), self.repos.generate_revision_id(2, "trunk", "trunk0")), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), self.repos.generate_revision_id(3, "trunk", "trunk0"))}, self.repos.get_fileid_map(3, "trunk", scheme))
 
     def test_copy(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        scheme = TrunkBranchingScheme()
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
         self.client_commit("dc", "Msg")
@@ -323,11 +347,12 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.client_copy("dc/trunk/file", "dc/trunk/bar")
         self.client_commit("dc", "Msg")
         self.assertEqual({
-            "": (ROOT_ID, self.repos.generate_revision_id(3, "trunk")), 
-            "bar": (generate_svn_file_id(self.repos.uuid, 3, "trunk", "bar"), self.repos.generate_revision_id(3, "trunk")), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), self.repos.generate_revision_id(2, "trunk"))}, self.repos.get_fileid_map(3, "trunk"))
+            "": (generate_svn_file_id(self.repos.uuid, 1, "trunk", ""), self.repos.generate_revision_id(3, "trunk", "trunk0")), 
+            "bar": (generate_svn_file_id(self.repos.uuid, 3, "trunk", "bar"), self.repos.generate_revision_id(3, "trunk", "trunk0")), "file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "file"), self.repos.generate_revision_id(2, "trunk", "trunk0"))}, self.repos.get_fileid_map(3, "trunk", scheme))
 
     def test_copy_nested_modified(self):
-        self.repos.set_branching_scheme(TrunkBranchingScheme())
+        scheme = TrunkBranchingScheme()
+        self.repos.set_branching_scheme(scheme)
         self.build_tree({"dc/trunk": None})
         self.client_add("dc/trunk")
         self.client_commit("dc", "Msg")
@@ -338,14 +363,14 @@ class GetMapTests(TestCaseWithSubversionRepository):
         self.build_tree({"dc/trunk/bar/file": "data2"})
         self.client_commit("dc", "Msg")
         self.assertEqual({
-          "": (ROOT_ID, 
-            self.repos.generate_revision_id(3, "trunk")), 
+          "": (generate_svn_file_id(self.repos.uuid, 1, "trunk", ""), 
+            self.repos.generate_revision_id(3, "trunk", "trunk0")), 
           "dir": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "dir"), 
-                self.repos.generate_revision_id(2, "trunk")),
+                self.repos.generate_revision_id(2, "trunk", "trunk0")),
           "dir/file": (generate_svn_file_id(self.repos.uuid, 2, "trunk", "dir/file"), 
-              self.repos.generate_revision_id(2, "trunk")),
+              self.repos.generate_revision_id(2, "trunk", "trunk0")),
           "bar": (generate_svn_file_id(self.repos.uuid, 3, "trunk", "bar"), 
-              self.repos.generate_revision_id(3, "trunk")),
+              self.repos.generate_revision_id(3, "trunk", "trunk0")),
           "bar/file": (generate_svn_file_id(self.repos.uuid, 3, "trunk", "bar/file"), 
-              self.repos.generate_revision_id(3, "trunk"))},
-            self.repos.get_fileid_map(3, "trunk"))
+              self.repos.generate_revision_id(3, "trunk", "trunk0"))},
+            self.repos.get_fileid_map(3, "trunk", scheme))
