@@ -14,13 +14,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.bzrdir import BzrDir
+"""Log walker tests."""
+
 from bzrlib.errors import NoSuchRevision
-from bzrlib.inventory import Inventory
 
 import os
 import logwalker
-from scheme import NoBranchingScheme, TrunkBranchingScheme
 from tests import TestCaseWithSubversionRepository
 from transport import SvnRaTransport
 
@@ -42,7 +41,7 @@ class TestLogWalker(TestCaseWithSubversionRepository):
 
         walker = logwalker.LogWalker(transport=SvnRaTransport(repos_url))
 
-        self.assertEqual(1, len(list(walker.follow_path("", 1))))
+        self.assertEqual(2, len(list(walker.follow_path("", 1))))
 
     def test_get_revision_paths(self):
         repos_url = self.make_client("a", "dc")
@@ -59,6 +58,11 @@ class TestLogWalker(TestCaseWithSubversionRepository):
         walker = logwalker.LogWalker(SvnRaTransport(repos_url))
         self.assertEqual({'': ('A', None, -1)}, walker.get_revision_paths(0))
 
+    def test_get_revision_paths_invalid(self):
+        repos_url = self.make_client("a", "dc")
+        walker = logwalker.LogWalker(SvnRaTransport(repos_url))
+        self.assertRaises(NoSuchRevision, lambda: walker.get_revision_paths(42))
+
     def test_get_branch_invalid_revision(self):
         repos_url = self.make_client("a", "dc")
         walker = logwalker.LogWalker(transport=SvnRaTransport(repos_url))
@@ -74,7 +78,7 @@ class TestLogWalker(TestCaseWithSubversionRepository):
 
         walker = logwalker.LogWalker(transport=SvnRaTransport(repos_url))
 
-        self.assertEqual(1, len(list(walker.follow_path("", 1))))
+        self.assertEqual(2, len(list(walker.follow_path("", 1))))
 
     def test_branch_log_specific(self):
         repos_url = self.make_client("a", "dc")
@@ -128,7 +132,7 @@ class TestLogWalker(TestCaseWithSubversionRepository):
         self.client_add("dc/branches")
         self.client_add("dc/tags")
         self.client_commit("dc", "My Message")
-        self.client_copy("dc/branches/tmp", "dc/tags/tmp");
+        self.client_copy("dc/branches/tmp", "dc/tags/tmp")
         self.client_commit("dc", "My Message2")
 
         walker = logwalker.LogWalker(transport=SvnRaTransport(repos_url))
@@ -141,7 +145,7 @@ class TestLogWalker(TestCaseWithSubversionRepository):
         self.client_add("dc/branches")
         self.client_add("dc/tags")
         self.client_commit("dc", "My Message")
-        self.client_copy("dc/branches/tmp", "dc/tags/tmp");
+        self.client_copy("dc/branches/tmp", "dc/tags/tmp")
         self.client_commit("dc", "My Message2")
         self.client_update("dc")
         self.client_set_prop("dc/tags", "myprop", "mydata")
@@ -273,9 +277,9 @@ class TestLogWalker(TestCaseWithSubversionRepository):
         self.client_commit("dc", "My Message")
 
         for (branch, paths, rev) in walker.follow_path("", 1):
-           self.assertEqual(branch, "")
-           self.assertTrue(paths.has_key("foo"))
-           self.assertEqual(rev, 1)
+            self.assertEqual(branch, "")
+            self.assertTrue(rev == 0 or paths.has_key("foo"))
+            self.assertTrue(rev in (0,1))
 
     def test_follow_history_nohist(self):
         repos_url = self.make_client("a", "dc")
@@ -293,9 +297,9 @@ class TestLogWalker(TestCaseWithSubversionRepository):
         self.client_commit("dc", "My Message")
 
         for (branch, paths, rev) in walker.follow_path("", 1):
-           self.assertEqual(branch, "")
-           self.assertTrue(paths.has_key("foo"))
-           self.assertEqual(rev, 1)
+            self.assertEqual(branch, "")
+            self.assertTrue(rev == 0 or paths.has_key("foo"))
+            self.assertTrue(rev in (0,1))
 
         iter = walker.follow_path("", 2)
         self.assertRaises(NoSuchRevision, list, iter)
@@ -404,6 +408,17 @@ class TestLogWalker(TestCaseWithSubversionRepository):
 
         self.assertEqual(("trunk", 1), walker.get_previous("anotherfile", 2))
 
+    def test_get_revision_info_zero(self):
+        repos_url = self.make_client("a", "dc")
+
+        walker = logwalker.LogWalker(transport=SvnRaTransport(repos_url))
+
+        info = walker.get_revision_info(0)
+
+        self.assertEqual(None, info[0])
+        self.assertEqual(None, info[1])
+        self.assertEqual(None, info[2])
+
     def test_get_revision_info(self):
         repos_url = self.make_client("a", "dc")
         self.build_tree({'dc/trunk/afile': "data"})
@@ -501,3 +516,19 @@ class TestLogWalker(TestCaseWithSubversionRepository):
                           'trunk/data/fg', 'trunk/data/fg/f1', 'trunk/db',
                           'trunk/db/f1', 'trunk/db/f2']), 
                 set(walker.find_children("trunk", 3)))
+
+    def test_fetch_property_change_only_trunk(self):
+        repos_url = self.make_client('d', 'dc')
+        self.build_tree({'dc/trunk/bla': "data"})
+        self.client_add("dc/trunk")
+        self.client_commit("dc", "My Message")
+        self.client_set_prop("dc/trunk", "some:property", "some data\n")
+        self.client_commit("dc", "My 3")
+        self.client_set_prop("dc/trunk", "some2:property", "some data\n")
+        self.client_commit("dc", "My 2")
+        self.client_set_prop("dc/trunk", "some:property", "some data4\n")
+        self.client_commit("dc", "My 4")
+        walker = logwalker.LogWalker(transport=SvnRaTransport(repos_url))
+        self.assertEquals({'trunk': ('M', None, -1)}, walker.get_revision_paths(3))
+
+
