@@ -14,8 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+"""Subversion transport tests."""
+
 from tests import TestCaseWithSubversionRepository
-from bzrlib.bzrdir import BzrDir
 from bzrlib.errors import NotBranchError, NoSuchFile, FileExists
 from transport import SvnRaTransport, bzr_to_svn_url
 from unittest import TestCase
@@ -24,7 +25,8 @@ import os
 
 class SvnRaTest(TestCaseWithSubversionRepository):
     def test_open_nonexisting(self):
-        self.assertRaises(NotBranchError, SvnRaTransport, "svn+nonexisting://foo/bar")
+        self.assertRaises(NotBranchError, SvnRaTransport, 
+                          "svn+nonexisting://foo/bar")
 
     def test_create(self):
         repos_url = self.make_client('a', 'ac')
@@ -45,6 +47,18 @@ class SvnRaTest(TestCaseWithSubversionRepository):
         t.reparent("%s/foo" % repos_url)
         self.assertEqual("%s/foo" % repos_url, t.base)
 
+    def test_lock_read(self):
+        repos_url = self.make_client('d', 'dc')
+        t = SvnRaTransport(repos_url)
+        lock = t.lock_read(".")
+        lock.unlock()
+
+    def test_lock_write(self):
+        repos_url = self.make_client('d', 'dc')
+        t = SvnRaTransport(repos_url)
+        lock = t.lock_write(".")
+        lock.unlock()
+
     def test_listable(self):
         repos_url = self.make_client('d', 'dc')
         t = SvnRaTransport(repos_url)
@@ -135,7 +149,7 @@ class SvnRaTest(TestCaseWithSubversionRepository):
         t.mkdir("bla")
         self.assertRaises(FileExists, t.mkdir, "bla")
 
-    def test_clone(self):
+    def test_clone2(self):
         repos_url = self.make_client('d', 'dc')
         self.build_tree({"dc/dir": None, "dc/bl": "data"})
         self.client_add("dc/dir")
@@ -155,6 +169,11 @@ class SvnRaTest(TestCaseWithSubversionRepository):
         t = SvnRaTransport("%s/dir" % repos_url)
         root = t.get_repos_root()
         self.assertEqual(repos_url, root)
+
+    def test_local_abspath(self):
+        repos_url = self.make_client('d', 'dc')
+        t = SvnRaTransport("%s" % repos_url)
+        self.assertEquals(os.path.join(self.test_dir, "d"), t.local_abspath('.'))
  
 
 class UrlConversionTest(TestCase):
