@@ -14,9 +14,8 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.bzrdir import BzrDir
-from bzrlib.errors import NoRepositoryPresent
-from bzrlib.tests import TestCase
+"""Basis and revision tree tests."""
+
 from bzrlib.workingtree import WorkingTree
 
 from tree import SvnBasisTree
@@ -24,40 +23,40 @@ from tests import TestCaseWithSubversionRepository
 
 class TestBasisTree(TestCaseWithSubversionRepository):
     def test_executable(self):
-        repos_url = self.make_client("d", "dc")
+        self.make_client("d", "dc")
         self.build_tree({"dc/file": "x"})
         self.client_add("dc/file")
         self.client_set_prop("dc/file", "svn:executable", "*")
         self.client_commit("dc", "executable")
-        tree = SvnBasisTree(WorkingTree.open("dc"))
+        tree = SvnBasisTree(self.open_checkout("dc"))
         self.assertTrue(tree.inventory[tree.inventory.path2id("file")].executable)
 
     def test_executable_changed(self):
-        repos_url = self.make_client("d", "dc")
+        self.make_client("d", "dc")
         self.build_tree({"dc/file": "x"})
         self.client_add("dc/file")
         self.client_commit("dc", "executable")
         self.client_update("dc")
         self.client_set_prop("dc/file", "svn:executable", "*")
-        tree = SvnBasisTree(WorkingTree.open("dc"))
+        tree = SvnBasisTree(self.open_checkout("dc"))
         self.assertFalse(tree.inventory[tree.inventory.path2id("file")].executable)
 
     def test_symlink(self):
-        repos_url = self.make_client("d", "dc")
+        self.make_client("d", "dc")
         import os
         os.symlink("target", "dc/file")
         self.build_tree({"dc/file": "x"})
         self.client_add("dc/file")
         self.client_commit("dc", "symlink")
         self.client_update("dc")
-        tree = SvnBasisTree(WorkingTree.open("dc"))
+        tree = SvnBasisTree(self.open_checkout("dc"))
         self.assertEqual('symlink', 
                          tree.inventory[tree.inventory.path2id("file")].kind)
         self.assertEqual("target",
                          tree.inventory[tree.inventory.path2id("file")].symlink_target)
 
     def test_symlink_next(self):
-        repos_url = self.make_client("d", "dc")
+        self.make_client("d", "dc")
         import os
         os.symlink("target", "dc/file")
         self.build_tree({"dc/file": "x", "dc/bla": "p"})
@@ -67,21 +66,21 @@ class TestBasisTree(TestCaseWithSubversionRepository):
         self.build_tree({"dc/bla": "pa"})
         self.client_commit("dc", "change")
         self.client_update("dc")
-        tree = SvnBasisTree(WorkingTree.open("dc"))
+        tree = SvnBasisTree(self.open_checkout("dc"))
         self.assertEqual('symlink', 
                          tree.inventory[tree.inventory.path2id("file")].kind)
         self.assertEqual("target",
                          tree.inventory[tree.inventory.path2id("file")].symlink_target)
 
     def test_executable_link(self):
-        repos_url = self.make_client("d", "dc")
+        self.make_client("d", "dc")
         import os
         os.symlink("target", "dc/file")
         self.build_tree({"dc/file": "x"})
         self.client_add("dc/file")
         self.client_set_prop("dc/file", "svn:executable", "*")
         self.client_commit("dc", "exe1")
-        wt = WorkingTree.open("dc")
+        wt = self.open_checkout("dc")
         tree = SvnBasisTree(wt)
         self.assertFalse(tree.inventory[tree.inventory.path2id("file")].executable)
         self.assertFalse(wt.inventory[wt.inventory.path2id("file")].executable)
